from django.urls import path

from mapbox_baselayer import views

app_name = 'mapbox_baselayer'

urlpatterns = [
    path('mapbox-baselayers/<int:pk>/tilejson/', views.MapboxBaseLayerJsonDetailView.as_view(), name='tilejson')
]



from django.core.management import BaseCommand
from django.utils.translation import gettext_lazy as _
from mapbox_baselayer.models import MapBaseLayer


class Command(BaseCommand):
    help = _("Install a MapBox base layer")

    def add_arguments(self, parser):
        parser.add_argument(
            '--mapbox-url',
            action='store',
            help='Mapbox url to use',
            default='mapbox://styles/mapbox/streets-v11'
        )

    def handle(self, *args, **options):
        MapBaseLayer.objects.create(
            name="Mapbox",
            base_layer_type="mapbox",
            tile_size=512,
            min_zoom=0,
            max_zoom=22,
            attribution='© Mapbox',
            map_box_url=options.get('mapbox_url')
        )
        self.stdout.write(self.style.SUCCESS(_("Mapbox base layer has been created.")))



from django.core.management import BaseCommand, CommandError
from django.utils.translation import gettext_lazy as _
from mapbox_baselayer.models import MapBaseLayer, BaseLayerTile


class Command(BaseCommand):
    help = _("Install an IGN base layer")
    layers = {
        'ortho': {"name": 'ORTHOIMAGERY.ORTHOPHOTOS', 'format': 'jpeg'},
        'plan': {"name": "GEOGRAPHICALGRIDSYSTEMS.PLANIGNV2", 'format': 'png'},
        'maps': {"name": 'GEOGRAPHICALGRIDSYSTEMS.MAPS', 'format': 'jpeg'},
        'se-classique': {"name": 'GEOGRAPHICALGRIDSYSTEMS.MAPS.SCAN-EXPRESS.CLASSIQUE', 'format': 'jpeg'},
        'se-standard': {"name": 'GEOGRAPHICALGRIDSYSTEMS.MAPS.SCAN-EXPRESS.STANDARD', 'format': 'jpeg'},
        'cadastre': {"name": 'CADASTRALPARCELS.PARCELS', 'format': 'png'},
    }

    def add_arguments(self, parser):
        parser.add_argument('key', type=str)
        parser.add_argument('--layers', nargs='+', type=str, default=['ortho', ])

    def handle(self, *args, **options):
        key = options.get('key')

        for layer in options.get('layers'):
            if layer not in self.layers:
                raise CommandError(_(f"'{layer}' is not a valid value. Should be '{', '.join(self.layers.keys())}'"))

        for layer in options.get('layers'):
            base_url = (
                f"//wxs.ign.fr/{key}/geoportail/wmts?LAYER={self.layers[layer]['name']}&EXCEPTIONS=text/xml&"
                f"FORMAT=image/{self.layers[layer]['format']}"
                f"&SERVICE=WMTS&VERSION=1.0.0&REQUEST=GetTile&STYLE=normal&TILEMATRIXSET=PM&"
                "TILEMATRIX={z}&TILEROW={y}&TILECOL={x}"
            )
            base_layer = MapBaseLayer.objects.create(
                name=f"IGN {layer}",
                base_layer_type="raster",
                tile_size=256,
                min_zoom=0,
                max_zoom=19,
                attribution='© IGN - GeoPortail'
            )
            BaseLayerTile.objects.bulk_create([
                BaseLayerTile(base_layer=base_layer, url=base_url),
            ])
        self.stdout.write(self.style.SUCCESS(_("IGN layer(s) created.")))



from django.core.management import call_command
from django.test import TestCase
from mapbox_baselayer.models import MapBaseLayer, BaseLayerTile


class InstallOpenTopoMapCommand(TestCase):
    @classmethod
    def setUpTestData(cls):
        call_command('install_opentopomap_baselayer')

    def test_base_layer_is_present(self):
        self.assertTrue(MapBaseLayer.objects.filter(name="OpenTopoMap").exists())

    def test_tile_are_present_and_differents(self):
        tiles = BaseLayerTile.objects.all()
        self.assertEqual(len(tiles), 3)

        self.assertEqual(len(set(tiles.values_list('url', flat=True))), 3)


class InstallOSMCommand(TestCase):
    @classmethod
    def setUpTestData(cls):
        call_command('install_osm_baselayer')

    def test_base_layer_is_present(self):
        self.assertTrue(MapBaseLayer.objects.filter(name="OSM").exists())

    def test_tile_are_present_and_differents(self):
        tiles = BaseLayerTile.objects.all()
        self.assertEqual(len(tiles), 3)

        self.assertEqual(len(set(tiles.values_list('url', flat=True))), 3)


class InstallMapboxCommand(TestCase):
    @classmethod
    def setUpTestData(cls):
        call_command('install_mapbox_baselayer')

    def test_without_arguments(self):
        self.assertTrue(MapBaseLayer.objects.filter(name="Mapbox").exists())


class InstallIGNCommand(TestCase):
    @classmethod
    def setUpTestData(cls):
        call_command('install_ign_baselayer', 'mykey', '--layers', 'ortho')

    def test_base_layer_is_present(self):
        self.assertTrue(MapBaseLayer.objects.filter(name="IGN ortho").exists())



# Generated by Django 2.2.7 on 2019-12-02 18:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mapbox_baselayer', '0004_auto_20191202_1834'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mapbaselayer',
            name='tile_size',
            field=models.PositiveSmallIntegerField(default=512, help_text='Raster tile size. Set 256 for 3rd party raster tilesets.'),
        ),
    ]



from django.contrib import admin

from . import models


class TileInLine(admin.StackedInline):
    model = models.BaseLayerTile
    extra = 0


@admin.register(models.MapBaseLayer)
class MapBaseLayerAdmin(admin.ModelAdmin):
    list_display = ('pk', 'name', 'slug')
    inlines = [TileInLine, ]



# Generated by Django 2.2.7 on 2019-12-02 18:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mapbox_baselayer', '0003_auto_20191202_0937'),
    ]

    operations = [
        migrations.AddField(
            model_name='mapbaselayer',
            name='attribution',
            field=models.CharField(blank=True, default='', max_length=255),
        ),
        migrations.AddField(
            model_name='mapbaselayer',
            name='tile_size',
            field=models.PositiveSmallIntegerField(default=512),
        ),
    ]



# Generated by Django 2.2.7 on 2019-12-02 09:37

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('mapbox_baselayer', '0002_auto_20191129_0942'),
    ]

    operations = [
        migrations.AlterField(
            model_name='baselayertile',
            name='base_layer',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='tiles', to='mapbox_baselayer.MapBaseLayer'),
        ),
    ]



from django.test import TestCase
from django.urls import reverse

from mapbox_baselayer.models import MapBaseLayer, BaseLayerTile


class MapBaseLayerViewTestCase(TestCase):
    def setUp(self) -> None:
        self.raster_base_layer = MapBaseLayer.objects.create(
            name='Raster layer',
            base_layer_type='raster',
            sprite='http://mystyle',
            glyphs='http://mystyle',
        )
        self.tile = BaseLayerTile.objects.create(
            base_layer=self.raster_base_layer,
            url='http://tiles/{x}/{y]/{z}'
        )
        self.mapbox_base_layer = MapBaseLayer.objects.create(
            name='Mapbox layer',
            order=0,
            base_layer_type='mapbox',
            map_box_url='mapbox://mystyle',
        )

    def test_tilejson_raster(self):
        self.maxDiff = None
        response = self.client.get(reverse('mapbox_baselayer:tilejson', args=(self.raster_base_layer.pk,)))
        self.assertEqual(response.status_code, 200)
        expected = {
            'layers': [
                {'id': 'raster-layer-background',
                 'source': 'raster-layer',
                 'type': 'raster'}],
            'sources': {'raster-layer': {
                'maxzoom': 22,
                'minzoom': 0,
                'tiles': ['http://tiles/{x}/{y]/{z}'],
                'type': 'raster',
                'attribution': '',
                'tileSize': 512, }},
            'version': 8,
            'sprite': 'http://mystyle',
            'glyphs': 'http://mystyle',
        }
        self.assertDictEqual(response.json(), expected)

    def test_tilejson_mapbox(self):
        self.maxDiff = None
        response = self.client.get(reverse('mapbox_baselayer:tilejson', args=(self.mapbox_base_layer.pk,)))
        self.assertEqual(response.status_code, 404)



from django.http import JsonResponse
from django.views.generic.detail import BaseDetailView


from mapbox_baselayer import models


class MapboxBaseLayerJsonDetailView(BaseDetailView):
    queryset = models.MapBaseLayer.objects.exclude(base_layer_type='mapbox')  # mapbox provide its own json

    def get(self, request, *args, **kwargs):
        return JsonResponse(self.get_object().tilejson)



# Generated by Django 4.1.3 on 2022-11-09 08:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mapbox_baselayer", "0005_auto_20191202_1840"),
    ]

    operations = [
        migrations.AlterField(
            model_name="mapbaselayer",
            name="map_box_url",
            field=models.CharField(
                blank=True,
                help_text="Mapbox or tilejson URL, starting ",
                max_length=255,
            ),
        ),
    ]



from django.test import TestCase
from django.urls import reverse

from mapbox_baselayer.models import MapBaseLayer


class MapBaseLayerTEstCase(TestCase):
    def setUp(self):
        self.mapbox_base_layer = MapBaseLayer.objects.create(
            name='Base layer 1',
            order=0,
            base_layer_type='mapbox',
            map_box_url='mapbox://mystyle',
            sprite='mapbox://mystyle',
            glyphs='mapbox://mystyle',
        )
        self.raster_base_layer = MapBaseLayer.objects.create(
            name='Raster layer',
            base_layer_type='raster',
            sprite='http://mystyle',
            glyphs='http://mystyle',
        )

    def test_str(self):
        self.assertEqual(self.mapbox_base_layer.name, str(self.mapbox_base_layer))

    def test_mapbox_url(self):
        self.assertEqual(self.mapbox_base_layer.url, self.mapbox_base_layer.map_box_url)

    def test_raster_url(self):
        self.assertEqual(self.raster_base_layer.url,
                         reverse('mapbox_baselayer:tilejson',
                                 args=(self.raster_base_layer.pk, )))



# Generated by Django 2.2.7 on 2019-11-25 14:28

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MapBaseLayer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, unique=True)),
                ('order', models.PositiveSmallIntegerField(default=0)),
                ('slug', models.SlugField(editable=False, unique=True)),
                ('base_layer_type', models.CharField(choices=[('mapbox', 'MapBox'), ('raster', 'Raster'), ('vector', 'Vector')], db_index=True, max_length=25)),
                ('map_box_url', models.CharField(blank=True, max_length=255)),
                ('sprite', models.CharField(blank=True, max_length=255)),
                ('glyphs', models.CharField(blank=True, max_length=255)),
                ('min_zoom', models.PositiveSmallIntegerField(default=0)),
                ('max_zoom', models.PositiveSmallIntegerField(default=22)),
            ],
            options={
                'verbose_name': 'Map base layers',
                'verbose_name_plural': 'Map base layers',
                'ordering': ('order', 'name'),
            },
        ),
        migrations.CreateModel(
            name='BaseLayerTile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.CharField(max_length=255)),
                ('base_layer', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='tiles', to='mapbox_baselayer.MapBaseLayer')),
            ],
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class MapboxBaselayerConfig(AppConfig):
    default_auto_field = "django.db.models.AutoField"
    name = 'mapbox_baselayer'
    verbose_name = _('MapBox Utils')



from django.core.management import BaseCommand
from django.utils.translation import gettext_lazy as _
from mapbox_baselayer.models import MapBaseLayer, BaseLayerTile


class Command(BaseCommand):
    help = _("Install an OSM base layer")

    def handle(self, *args, **options):
        base_layer = MapBaseLayer.objects.create(
            name="OSM",
            base_layer_type="raster",
            tile_size=256,
            min_zoom=0,
            max_zoom=19,
            attribution='<a href="https://www.openstreetmap.org/copyright">OSM Contributors</a>'
        )
        BaseLayerTile.objects.bulk_create([
            BaseLayerTile(base_layer=base_layer, url="//a.tile.openstreetmap.org/{z}/{x}/{y}.png"),
            BaseLayerTile(base_layer=base_layer, url="//b.tile.openstreetmap.org/{z}/{x}/{y}.png"),
            BaseLayerTile(base_layer=base_layer, url="//c.tile.openstreetmap.org/{z}/{x}/{y}.png"),
        ])
        self.stdout.write(self.style.SUCCESS(_("OSM base layer has been created.")))



from django.core.management import BaseCommand
from django.utils.translation import gettext_lazy as _
from mapbox_baselayer.models import MapBaseLayer, BaseLayerTile


class Command(BaseCommand):
    help = _("Install an OpenTopoMap base layer")

    def handle(self, *args, **options):
        base_layer = MapBaseLayer.objects.create(
            name="OpenTopoMap",
            base_layer_type="raster",
            tile_size=256,
            min_zoom=2,
            max_zoom=17,
            attribution='map data: © <a href="https://openstreetmap.org/copyright">OpenStreetMap</a> contributors,'
                        '<a href="http://viewfinderpanoramas.org">SRTM</a> | map style: © '
                        '<a href="https://opentopomap.org">OpenTopoMap</a> '
                        '(<a href="https://creativecommons.org/licenses/by-sa/3.0/">CC-BY-SA</a>)'
        )
        BaseLayerTile.objects.bulk_create([
            BaseLayerTile(base_layer=base_layer, url="//a.tile.opentopomap.org/{z}/{x}/{y}.png"),
            BaseLayerTile(base_layer=base_layer, url="//b.tile.opentopomap.org/{z}/{x}/{y}.png"),
            BaseLayerTile(base_layer=base_layer, url="//c.tile.opentopomap.org/{z}/{x}/{y}.png"),
        ])
        self.stdout.write(self.style.SUCCESS(_("OpenTopoMap base layer has been created.")))



# Generated by Django 3.2.20 on 2023-07-11 12:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mapbox_baselayer', '0006_alter_baselayertile_id_alter_mapbaselayer_id_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mapbaselayer',
            name='attribution',
            field=models.CharField(blank=True, default='', max_length=1024),
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-29 09:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mapbox_baselayer', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='baselayertile',
            name='url',
            field=models.CharField(max_length=512),
        ),
    ]



from django.db import models
from django.urls import reverse
from django.utils.functional import cached_property
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _


class MapBaseLayer(models.Model):
    BASE_LAYER_TYPES = (
        ('mapbox', 'MapBox'),
        ('raster', 'Raster'),
        ('vector', 'Vector'),
    )
    name = models.CharField(max_length=50, unique=True)
    order = models.PositiveSmallIntegerField(default=0)
    slug = models.SlugField(unique=True, editable=False)
    base_layer_type = models.CharField(max_length=25, choices=BASE_LAYER_TYPES, db_index=True, blank=False)
    map_box_url = models.CharField(max_length=255, blank=True, help_text=_("Mapbox or tilejson URL, starting "))
    sprite = models.CharField(max_length=255, blank=True)
    glyphs = models.CharField(max_length=255, blank=True)
    min_zoom = models.PositiveSmallIntegerField(default=0)
    max_zoom = models.PositiveSmallIntegerField(default=22)
    tile_size = models.PositiveSmallIntegerField(
        default=512, help_text=_("Raster tile size. Set 256 for 3rd party raster tilesets.")
    )
    attribution = models.CharField(max_length=1024, blank=True, default="")

    def __str__(self):
        return self.name

    def get_source(self):
        source = {
            "type": f"{self.base_layer_type}",
            "tiles": list(self.tiles.values_list('url', flat=True)),
            "minzoom": self.min_zoom,
            "maxzoom": self.max_zoom,
            "attribution": self.attribution,
        }

        if self.base_layer_type == 'raster':
            # only available for raster layers
            source["tileSize"] = self.tile_size

        return source

    @cached_property
    def tilejson(self):
        data = {
            "version": 8,
            "sources": {
                f"{self.slug}": self.get_source(),
            },
            "layers": [
                {"id": f"{self.slug}-background",
                 "type": f"{self.base_layer_type}",
                 "source": f"{self.slug}"}
            ]
        }
        # prevents mapbox problems by set glyphs and sprite only if specified
        if self.sprite:
            data["sprite"] = self.sprite

        data["glyphs"] = self.glyphs or "mapbox://fonts/mapbox/{fontstack}/{range}.pbf"

        return data

    @cached_property
    def url(self):
        if self.base_layer_type != 'mapbox':
            return reverse('mapbox_baselayer:tilejson', args=(self.pk, ))
        else:
            return self.map_box_url

    @cached_property
    def real_url(self):
        if self.base_layer_type not in ('mapbox', 'vector'):
            return self.url
        else:
            return self.map_box_url.replace("mapbox://styles",
                                            "https://api.mapbox.com/styles/v1")

    def save(self, *args, **kwargs):
        self.slug = slugify(self.name)
        super().save(*args, **kwargs)

    class Meta:
        verbose_name = _("Map base layers")
        verbose_name_plural = _("Map base layers")
        ordering = (
            'order', 'name'
        )


class BaseLayerTile(models.Model):
    base_layer = models.ForeignKey(MapBaseLayer, related_name='tiles', on_delete=models.CASCADE)
    url = models.CharField(max_length=512)



#!/usr/bin/env python

import os

from setuptools import setup, find_packages

HERE = os.path.abspath(os.path.dirname(__file__))

README = open(os.path.join(HERE, 'README.md')).read()
CHANGES = open(os.path.join(HERE, 'CHANGES.md')).read()

tests_require = [
    'factory-boy',
    'flake8',
    'coverage',
]

setup(
    name='django-mapbox-baselayer',
    version=open(os.path.join(HERE, 'mapbox_baselayer', 'VERSION.md')).read().strip(),
    include_package_data=True,
    author="Makina Corpus",
    author_email="geobi@makina-corpus.com",
    description='Django model to store and serve mapbox base layer config',
    long_description=README + '\n\n' + CHANGES,
    description_content_type="text/markdown",
    long_description_content_type="text/markdown",
    packages=find_packages(),
    url='https://github.com/makinacorpus/django-mapbox-baselayer.git',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Framework :: Django :: 3.2',
        'Framework :: Django :: 4.0',
        'Framework :: Django :: 4.1',
    ],
    install_requires=[
        'django>=2.2',
    ],
    tests_require=tests_require,
    extras_require={
        'dev': tests_require + [
            'django-debug-toolbar',
        ]
    }
)
